# Copyright 2015 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to get IAM policy for a resource."""

from googlecloudsdk.api_lib.projects import util
from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class GetIamPolicy(util.ProjectCommand):
  """Get IAM policy for a project.

  Gets the IAM policy for a project, given a project ID.
  """

  detailed_help = {
      'EXAMPLES': """\
          The following command prints the IAM policy for a project with
          the ID `example-project-id-1`:

            $ {command} example-project-id-1
          """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument('id', metavar='PROJECT_ID',
                        completion_resource='cloudresourcemanager.projects',
                        list_command_path='projects',
                        help='ID for the project whose policy you want to get.')

  @util.HandleHttpError
  def Run(self, args):
    projects = util.GetClient()
    messages = util.GetMessages()

    project_ref = self.GetProject(args.id)
    policy_request = messages.CloudresourcemanagerProjectsGetIamPolicyRequest(
        resource=project_ref.Name(),
        getIamPolicyRequest=messages.GetIamPolicyRequest(),
    )
    return projects.projects.GetIamPolicy(policy_request)
